﻿namespace mfs {
    internal class TLGDatabase {
        private enum block_types {
            NONE,
            REFERENCE_DATA,
            BASIC_TABLE,
            INCREMENT_TABLE,
            DERIVATIVE_TABLE,
        } ;
        private block_types currentblock = block_types.NONE;

        public Dictionary<string, double> reference_values = new Dictionary<string, double>();

        public TLGDatabase(string path) {
            readfile(path);
        }

        private void readfile(string path) {
            StreamReader fs = File.OpenText(path);
            string line = "";

            while (!fs.EndOfStream) {
                line = fs.ReadLine();
                if (line == null) {
                    line = "";
                }
                if (string.IsNullOrWhiteSpace(line) || (line[0] == '%')) {
                    // lines starting with % are comment lines and ignored
                    // blank or whitespace lines also ignored
                } else if (line == "END BLOCK") {
                    currentblock = block_types.NONE;

                } else {
                    switch (currentblock) {

                        case block_types.NONE:
                            // look for data blocks, or an include line
                            switch (line) {
                                case "BEGIN REFERENCE DATA BLOCK":
                                    currentblock = block_types.REFERENCE_DATA;
                                    break;
                                case "BEGIN BASIC TABLE BLOCK":
                                    currentblock = block_types.BASIC_TABLE;
                                    break;
                                case "BEGIN INCREMENT TABLE BLOCK":
                                    currentblock = block_types.INCREMENT_TABLE;
                                    break;
                                case "BEGIN DERIVATIVE TABLE BLOCK":
                                    currentblock = block_types.DERIVATIVE_TABLE;
                                    break;
                            }

                            if (line.StartsWith("INCLUDE,", StringComparison.InvariantCultureIgnoreCase)) {
                                string includepath = line.Substring(8).Trim();
                                if (File.Exists(includepath)) {
                                    readfile(includepath);
                                } else {
                                    throw new Exception("Could not find included file " + includepath);
                                }
                            }
                            break;

                        case block_types.REFERENCE_DATA:
                            string[] splitline = line.Split(',', StringSplitOptions.TrimEntries & StringSplitOptions.RemoveEmptyEntries);
                            if (splitline.Length != 2) {
                                throw new Exception("Error parsing reference parameter " + splitline[0]);
                            } else {
                                double val = double.NaN;
                                if (double.TryParse(splitline[1], out val)) {
                                    reference_values.Add(splitline[0], val);
                                } else {
                                    throw new Exception("Could not parse reference parameter " + splitline[0]);
                                }

                            }
                            break;

                        case block_types.BASIC_TABLE:
                            break;

                        case block_types.INCREMENT_TABLE:
                            break;

                        case block_types.DERIVATIVE_TABLE:
                            break;

                        default:
                            break;
                    }
                }
            }
            if (currentblock != block_types.NONE) {
                throw new Exception("End of file encountered inside a block");
            }

            fs.Close();
        }
    }
}
